
function StellarUF:CreateUnitFrame(unit, size, reverse)
	local left, right, neg = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT"
	local rev = (not reverse and 1) or -1
	local lay = self.layout[size]
	local db = self.db.profile
	
	-- baseframe
	self.units[unit] = CreateFrame("BUTTON", "StellarUF"..unit, UIParent, "SecureUnitButtonTemplate")
	local f = self.units[unit]
	f.unit = unit
	f.frametype = size
	f.reversed = reverse
	f:SetWidth(lay.frame.w)
	f:SetHeight(lay.frame.h)
	
	f:SetBackdrop({ bgFile = self.surface:Fetch(self.db.profile.texture), })
	f:SetBackdropColor(0, 0, 0, 0.2)
	
	f:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", db.pos[unit].x, db.pos[unit].y)
	f:SetScale(db.scale[size] or 1.0)
	f:SetScript("OnEnter", function() UnitFrame_OnEnter() this:SetBackdropColor(1, 1, 0.5, 0.8) end)
	f:SetScript("OnLeave", function() UnitFrame_OnLeave() this:SetBackdropColor(0, 0, 0, 0.2) end)
	f:SetClampedToScreen(true)
	
	-- portrait
	self:CreatePortrait(f, lay.portrait.s, size)
	f.portrait:SetPoint("TOP"..left, f, "TOP"..left, lay.portrait.x * rev, lay.portrait.y)
	
	-- healthbar
	self:CreateHealthBar(f, lay.hp.w, lay.hp.h, lay.hp.fontsize, reverse, size)
	f.hp:SetPoint("TOP"..left, f.portrait, "TOP"..right, lay.hp.x * rev, lay.hp.y)
	
	-- powerbar
	self:CreateManaBar(f, lay.mp.w, lay.mp.h, lay.mp.fontsize, reverse, size)
	f.mp:SetPoint("TOP"..right, f.hp, "BOTTOM"..right, lay.mp.x * rev, lay.mp.y)
	
	-- create a bunch of info icons
	if size == "full" or size == "medium" then
		self:CreateIcon(f, "classicon", lay.classicon.s, true)
		f.classicon:EnableMouse(true)
		f.classicon:SetPoint("TOP"..right, f.mp, "TOP"..left, lay.classicon.x * rev, lay.classicon.y)
		f.classicon:SetScript("OnEnter", function() self:ClassIconOnEnter() end)
		f.classicon:SetScript("OnLeave", function() GameTooltip:Hide() end)
		
		if unit ~= "target" then
			self:CreateIcon(f, "statusicon", lay.statusicon.s)
			f.statusicon:SetPoint("CENTER", f.portrait, "BOTTOM"..left, lay.statusicon.x * rev, lay.statusicon.y)
		end
		
		self:CreateIcon(f, "pvpicon", lay.pvpicon.s)
		f.pvpicon:SetPoint("CENTER", f.portrait, left, (unit == "target" and lay.pvpicon.tarx) or lay.pvpicon.x, lay.pvpicon.y)
		
		self:CreateIcon(f, "leadericon", lay.leadericon.s)
		f.leadericon.texture:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
		f.leadericon:SetPoint("CENTER", f.portrait, "BOTTOM"..left, lay.leadericon.x * rev, lay.leadericon.y)
		
		self:CreateIcon(f, "looticon", lay.looticon.s)
		f.looticon.texture:SetTexture("Interface\\GroupFrame\\UI-Group-MasterLooter")
		f.looticon:SetPoint("CENTER", f.portrait, "BOTTOM"..left, lay.looticon.x * rev, lay.looticon.y)

		-- create debuff alert
		if self.cls ~= "WARRIOR" and self.cls ~= "ROGUE" and self.cls ~= "HUNTER" and self.cls ~= "WARLOCK" then
			self:CreateAuraIcon(f, "debuffalert", lay.frame.h * 0.8, nil, nil, true)
			local da = f.debuffalert
			da:SetFrameStrata("HIGH")
			da:SetPoint(right, f, right, rev * 2, 0)
			da.icon:ClearAllPoints()
			da.icon:SetPoint("TOPRIGHT", da, -4, -4)
			da.icon:SetPoint("BOTTOMLEFT", da, 2, 2)
			da.elapsed = 0
			da.alpha = 0
			da.sign = 1
			da:SetScript("OnUpdate", function() self:UpdateDebuffAlert() end)
			da:SetAlpha(0)
			da:Hide()
		end
	end
	
	-- create buff/debuff icons (not for player or tiny mode frames)
	if unit ~= "player" and lay.auraicon then
		local la = lay.auraicon
		self:CreateAuraGroup(f, "buff", la.s, la.bcount or 16, "buff", la.brows, la.bcols)
		f.buff1:SetPoint("TOPLEFT", f, "TOPLEFT", la.bx, la.by)
		
		-- make party debuffs bigger than normal
		local debuffsize = la.s
		if string.find(unit, "party") then 
			debuffsize = la.s + 4 
		end
		self:CreateAuraGroup(f, "debuff", debuffsize, la.dcount or 16, "debuff", la.drows, la.dcols)
		f.debuff1:SetPoint("TOPLEFT", f, "TOPLEFT", la.dx, la.dy)
	end
	
	-- create group number text on player/target frames
	if unit == "player" or unit == "target" then
		self:CreateFrame(f, "group", 13, 13, true, 0.3, 0.3, 0.3, 0.8)
		self:CreateText( f.group, "text", 13, 13, self.fontnum, 10, 
		                 "CENTER", "CENTER", "CENTER", 0, 0 )
		f.group:SetPoint("TOP", f.leadericon, "CENTER", 0, 0)
		f.group:Hide()
	end
	
	-- pet happiness icon for hunters (just recycled blizz's)
	if unit == "pet" and self.cls == "HUNTER" then
		PetFrameHappiness:SetParent(self.units.pet)
		PetFrameHappiness:ClearAllPoints()
		PetFrameHappiness:SetPoint("TOP", self.units.pet.portrait, "BOTTOM", 5, 0)
		PetFrameHappiness:Show()
		PetFrameHappiness:SetWidth(16)
		PetFrameHappiness:SetHeight(15.5)
		PetFrameHappiness:RegisterEvent("UNIT_HAPPINESS")
		PetFrameHappiness:SetScript("OnEvent", function() PetFrame_SetHappiness() end)
	end
	
	-- target only stuff
	if unit == "target" then self:TargetOnlyStuff() end
	
	-- puts the word "Focus" above the focus frame
	if unit == "focus" then
		self:CreateText( f, "title", 30, 10, self.fontnum, 8, 
		                 "CENTER", "BOTTOM", "BOTTOM", 0, 18, "ARTWORK" )
		f.title:SetText("|cffffff00Focus|r")
	end

	-- make focustarget a child of focus
	if unit == "focustarget" then
		f:ClearAllPoints()
		f:SetParent(self.units.focus or UIParent)
		f:SetPoint("LEFT", self.units.focus, "RIGHT", 1, 0)
	end

	f:SetAlpha(0.9)
	
	--Set Secure attributes
	if not self.dropdown[unit] then
		SecureUnitButton_OnLoad(f, unit)
	else
		SecureUnitButton_OnLoad(f, unit, function() ToggleDropDownMenu(1, nil, self.dropdown[unit], f, lay.frame.w / 2, 0) end)
	end
	if unit ~= "player" then
		f:Hide()
		RegisterUnitWatch(f, false)
	end
	
	-- this is click casting support? ok
	ClickCastFrames = ClickCastFrames or {}
	ClickCastFrames[f] = true
	
	f:SetScript("OnShow", function() self:UpdateUnit(this.unit, true) end)
end

-- just a chunk of code taken out of the above
function StellarUF:TargetOnlyStuff()
	local f = self.units.target
	
	--combo points
	if self.cls == "ROGUE" or self.cls == "DRUID" then
		self:CreateFrame(f, "combo", 36, 18)
		f.combo:SetFrameStrata("HIGH")
		self:CreateTexture(f.combo, "texture", 36, 18, "OVERLAY", 0)
		self:CreateTexture(f.combo, "glow", 36, 18, "ARTWORK", 0)
		f.combo.alpha = 0.5
		f.combo.elapsed = 0
		f.combo.sign = 1
		f.combo:SetPoint("BOTTOMLEFT", f.portrait, "BOTTOMLEFT", -6, -6)
		f.combo:RegisterEvent("PLAYER_TARGET_CHANGED")
		f.combo:RegisterEvent("PLAYER_COMBO_POINTS")
		f.combo:SetScript("OnEvent", function() self:UpdateCombo() end)
		f.combo:SetScript("OnUpdate", function() self:UpdateComboGlow() end)
		f.combo:Hide()
	end
	
	--raid target icon
	self:CreateIcon(f, "raidtarget", 16)
	f.raidtarget:SetPoint("CENTER", f.portrait, "TOPLEFT", 2, 2)
	f.raidtarget.texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	f.raidtarget.texture:SetAlpha(0.8)

	-- inspect button
	f.inspect = CreateFrame("BUTTON", nil, f)
	f.inspect:SetWidth(16)
	f.inspect:SetHeight(16)
	f.inspect:SetPoint("CENTER", f, "BOTTOMRIGHT", -1, 1)
	f.inspect:SetFrameStrata("HIGH")
	f.inspect:SetPushedTexture("Interface\\AddOns\\StellarUF\\images\\inspectdown")
	f.inspect:SetNormalTexture("Interface\\AddOns\\StellarUF\\images\\inspectup")
	f.inspect:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	f.inspect:SetScript("OnMouseUp", function() 
		if UnitIsVisible("target") then
			if arg1 == "LeftButton" then InspectUnit("target") 
			elseif arg1 == "RightButton" then 
				if not DressUpFrame:IsVisible() then ShowUIPanel(DressUpFrame) end
				DressUpModel:SetUnit("target")
			end 
		end
	end)
	f.inspect:SetScript("OnEnter", function() 
		GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
		GameTooltip:SetText("Inspect")
		GameTooltip:Show()
	end)
	f.inspect:SetScript("OnLeave", function() GameTooltip:Hide() end)
end

-- position
function StellarUF:SavePosition(unit)
	local a,b,c,d,e = self.units[unit]:GetPoint()
	if a == "TOPLEFT" and c == "TOPLEFT" then
		self.db.profile.pos[unit].x = math.floor(d + 0.5)
		self.db.profile.pos[unit].y = math.floor(e + 0.5)
	end
end

-- make movable or not
function StellarUF:ToggleDrag()
	if self.unlocked then
		for u,f in pairs(self.units) do
			if f.frametype ~= "tiny" and u ~= "focustarget" then
				f:SetMovable(false)
				f:RegisterForDrag()
			end
		end
		self.unlocked = false
	else
		for u,f in pairs(self.units) do
			if f.frametype ~= "tiny" and u ~= "focustarget" then
				f:SetMovable(true)
				f:RegisterForDrag("LeftButton")
				f:SetScript("OnDragStart", function() if not InCombatLockdown() then this:StartMoving() end end)
				f:SetScript("OnDragStop", function() this:StopMovingOrSizing() self:SavePosition(this.unit) end)
			end
		end
		self.unlocked = true
	end
end

