StellarUF = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceHook-2.1", "AceConsole-2.0", "Metrognome-2.0")

local _G = getfenv(0)

function StellarUF:OnInitialize()
	_,self.cls = UnitClass("player")
	self.units = { }
	self:LoadVariables()
	self:InitSpellRange()
	
	-- maybe unnecessary, but shouldn't hurt
	self:ScheduleLeaveCombatAction("LoadFrames")
end

function StellarUF:LoadFrames()
	self:HideBlizzStuff()

	self:CreateUnitFrame("player", "full")
	self:CreateUnitFrame("target", "full", true)
	self:CreateUnitFrame("targettarget", "small")
	self:CreateUnitFrame("focus", "small")
	self:CreateUnitFrame("focustarget", "small", true)
	self:CreateUnitFrame("pet", "medium")

	-- only care to create party frames dynamically
	if self:IsInGroup() then
		self:CreateParty()
	else
		self:RegisterEvent("PARTY_MEMBERS_CHANGED", "CreateParty")
	end
	
	self:Hook("RaidOptionsFrame_UpdatePartyFrames", function() self:RaidPartyUpdate() end, true)

	self:RegisterEvents()
	self:RegisterEvent("AceEvent_FullyInitialized", function() self:UpdateAll() end)
end

function StellarUF:LoadVariables()
	-- font for digit related text
	self.fontnum = "Interface\\AddOns\\StellarUF\\fonts\\font1.ttf"
	-- font for player names
	self.fontname = "Interface\\AddOns\\StellarUF\\fonts\\font1.ttf"
	-- font for player info
	self.fontinfo = "Interface\\AddOns\\StellarUF\\fonts\\font1.ttf"
	
	-- health bar color
	self.hpcolor = { r=0, g=0.7, b=0, }
	-- power bar color
	self.powercolor = {
		[0] = { r=0.1, g=0.1, b=0.7, }, 		-- mana
		[1] = { r=0.5, g=0.1, b=0.1, }, 		-- rage
		[2] = { r=0.8, g=0.4, b=0, }, 		-- focus
		[3] = { r=0.6, g=0.6, b=0, }, 		-- energy
		[4] = { r=0.5, g=0.5, b=0, }, 		-- happiness
	}
	
	self.aurabg = "Interface\\AddOns\\StellarUF\\images\\backdrop"
	self.combo = "Interface\\AddOns\\StellarUF\\images\\combo\\combo"
	
	-- reaction colors
	self.reactioncolor = {
		[1] = { r=0.85, g=0, b=0 },		-- Hostile
		[2] = { r=0.85, g=0, b=0 },		-- Hostile
		[3] = { r=0.85, g=0.5, b=0 },		-- Unfriendly
		[4] = { r=0.85, g=0.85, b=0 },	-- Neutral
		[5] = { r=0, g=0.95, b=0 },		-- Friendly
		[6] = { r=0, g=0.95, b=0 },		-- Friendly
		[7] = { r=0, g=0.95, b=0 },		-- Friendly
		[8] = { r=0.97, g=0.97, b=0.97 },	-- Something white
	}
	-- elite/rare text
	self.classificationtext = {
		worldboss = " Boss",
		rareelite = "+ Rare",
		elite = "+",
		rare = " Rare",
		normal = "",
	}
	self.formats = { 
		hptext = "%d%%\n%d/%d",
		mptext = "%d/%d",
		perc = "%d%%",
		deficit = "-%d\n%d/%d",
	}
	
	-- use blizz's default unit dropdown menus
	self.dropdown = {
		player = PlayerFrameDropDown,
		target = TargetFrameDropDown,
		party1 = PartyMemberFrame1DropDown,
		party2 = PartyMemberFrame2DropDown,
		party3 = PartyMemberFrame3DropDown,
		party4 = PartyMemberFrame4DropDown,
		pet = PetFrameDropDown,
	}

	-- basic layout parameters
	self.layout = {
		full = { -- Player and Target frames
			frame = { w = 191, h = 50, },
			portrait = { s = 48, x = 1, y = -1, },
			hp = { w = 140, h = 33, x = 1, y = -1, fontsize = 10, },
			mp = { w = 126, h = 12, x = 0, y = -1, fontsize = 10, },
			classicon = { s = 12, x = -2, y = 0, },
			statusicon = { s = 15, x = -2, y = 0, },
			pvpicon = { s = 42, x = 5, y = -8, tarx = 9, },
			leadericon = { s = 12, x = 13, y = 0, },
			looticon = { s = 10, x = 24, y = 1, },
			auraicon = { s = 16, bx = 2, by = -51, bcount = 32, brows = 2, bcols = 16, dx = 2, dy = -68, dcount = 40, drows = 3, dcols = 16, },
		},
		medium = { -- Pet and Party frames
			frame = { w = 151, h = 34, },
			portrait = { s = 32, x = 1, y = -1, },
			hp = { w = 116, h = 22, x = 1, y = -1, fontsize = 7, },
			mp = { w = 107, h = 8, x = 0, y = -1, fontsize = 7, },
			classicon = { s = 8, x = -1, y = 0, },
			statusicon = { s = 12, x = -1, y = 0, },
			pvpicon = { s = 32, x = 4, y = -6, },
			leadericon = { s = 9, x = 8, y = 0, },
			looticon = { s = 8, x = 16, y = 1, },
			auraicon = { s = 12, bx = 32, by = -34, dx = 152, dy = -1, dcount = 8, drows = 2, dcols = 4, },
		},
		small = { -- TargetOfTarget and Focus frames
			frame = { w = 60, h = 18, },
			portrait = { s = 16, x = 1, y = -1, },
			hp = { w = 42, h = 12, x = 0, y = 0, fontsize = 7, },
			mp = { w = 42, h = 4, x = 0, y = 0, },
			auraicon = { s = 8, bx = 0, by = -18, bcount = 8, dcount = 8, dx = 0, dy = -26, },
		},
		tiny = { -- Party pet frames
			frame = { w = 35, h = 12, },
			portrait = { s = 12, x = 0, y = 0, },
			hp = { w = 23, h = 8, x = 0, y = -1, fontsize = 6, },
			mp = { w = 23, h = 2, x = 0, y = 0, },
		},
	}
	
	self:RegisterDB("StellarUFDB")
	self:RegisterDefaults("profile", {
		pos = {
			player = { x = 21, y = -35, },
			target = { x = 230, y = -35, },
			pet = { x = 60, y = -85, },
			targettarget = { x = 430, y = -40, },
			party1 = { x = 16, y = -130, },
			party2 = { x = 16, y = -180, },
			party3 = { x = 16, y = -230, },
			party4 = { x = 16, y = -280, },
			partypet1 = { x = 0, y = 0, }, -- party pets have dummy vars, it's set to owner
			partypet2 = { x = 0, y = 0, },
			partypet3 = { x = 0, y = 0, },
			partypet4 = { x = 0, y = 0, },
			focus = { x = 430, y = -100, },
			focustarget = { x = 430, y = -100, },
		},
		scale = {
			full = 1.0,
			medium = 1.0,
			small = 1.0,
		},
		texture = "Charcoal Low",
		showmodels = true, 
		deficit = false, 
	})
	
	self.surface = AceLibrary("Surface-1.0")
	self.surface:Register("Flatsmooth", "Interface\\AddOns\\StellarUF\\images\\flatsmooth")
	self.surface:Register("Charcoal Low", "Interface\\AddOns\\StellarUF\\images\\charcoallow")
	self.surface:Register("Steel", "Interface\\AddOns\\StellarUF\\images\\steel")

	
	self.dewdrop = AceLibrary("Dewdrop-2.0")
	self:LoadOptions()
	self:RegisterChatCommand({ "/stellaruf"}, {
		type = "execute", name = "Stellar", desc = "Stellar",
		func = function() 		
			self.dewdrop:Open( UIParent,
		        'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end,
		        'cursorX', true, 'cursorY', true ) 
		end,
	})

end

function StellarUF:LoadOptions()
	local db = self.db.profile
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "StellarUF",
				order = 1,
			},
			texture = {
				type = "text",
				name = "Bar Texture",
				desc = "Statusbar textures",
				set = function(v) db.texture = v self:UpdateTexture() end,
				get = function() return db.texture end,
				validate = self.surface:List(),
				order = 2,
			},
			scale = {
				type = "group",
				name = "Scaling",
				desc = "Scale certain types of frames",
				order = 3,
				args = {
					full = {
						type = "range",
						name = "Scale full frames.",
						desc = "Player and Target",
						set = function(v) db.scale.full = v self:ScaleFrame("full") end,
						get = function() return db.scale.full end,
						min = 0.2, max = 3.0, step = 0.05,
						order = 1,
					},
					medium = {
						type = "range",
						name = "Scale medium frames.",
						desc = "Pet and Party",
						set = function(v) db.scale.medium = v self:ScaleFrame("medium") end,
						get = function() return db.scale.medium end,
						min = 0.2, max = 3.0, step = 0.05,
						order = 2,
					},
					small = {
						type = "range",
						name = "Scale small frames.",
						desc = "TargetOfTarget and Focus",
						set = function(v) db.scale.small = v self:ScaleFrame("small") end,
						get = function() return db.scale.small end,
						min = 0.2, max = 3.0, step = 0.05,
						order = 3,
					},
				},
			},
			model = {
				type = "toggle",
				name = "Toggle 3D Models",
				desc = "Toggle showing 3D/2D portraits",
				set = function() self:ToggleModels() end,
				get = function() return db.showmodels end,
				order = 4,
			},
			deficit = {
				type = "toggle",
				name = "Show health deficit",
				desc = "Show HP deficit instead of percent",
				set = function(v) db.deficit = v self:UpdateAll() end, 
				get = function() return db.deficit end, 
				order = 6,
			},
			lock = {
				type = "toggle",
				name = "Unlock",
				desc = "Unlock the frames so you can move them",
				set = function(v) self:ToggleDrag() end, 
				get = function() return self.unlocked end, 
				order = 7,
			},
		},
	}
	self:RegisterEvent("Surface_SetGlobal", "UpdateTexture")
end

-- update texture when user changes it
function StellarUF:UpdateTexture()
	local t = self.surface:Fetch(self.db.profile.texture)
	for u,f in pairs(self.units) do
		f:SetBackdrop({ bgFile=t, })
		f:SetBackdropColor(0, 0, 0, 0.2)
		f.hp.bar:SetTexture(t)
		f.hp:SetBackdrop({ bgFile = t, })
		f.hp:SetBackdropColor(0, self.hpcolor.g/3.5, 0, 0.6)
		if f.hp.backbar then
			f.hp.backbar:SetTexture(t)
		end
		f.mp.bar:SetTexture(t)
		f.mp:SetBackdrop({ bgFile = t, })
		f.mp:SetBackdropColor(0, 0, 0, 0.6)
		if f.mp.backbar then
			f.mp.backbar:SetTexture(t)
		end
	end
end

-- update scaling when user changes it
function StellarUF:ScaleFrame(type)
	local s = self.db.profile.scale[type]
	for u,f in pairs(self.units) do
		if f.frametype == type and u ~= "focustarget" then
			f:SetScale(s)
		end
	end
end

-- hide and unregister blizz defaults
function StellarUF:HideBlizzStuff()
	PlayerFrame:UnregisterAllEvents()
	PlayerFrameHealthBar:UnregisterAllEvents()
	PlayerFrameManaBar:UnregisterAllEvents()
	PlayerFrame:Hide()
	TargetFrame:UnregisterAllEvents()
	TargetFrame:Hide()
	TargetofTargetFrame:UnregisterAllEvents()
	TargetofTargetFrame:Hide()
	ComboFrame:UnregisterAllEvents()
	for i=1,4 do
		_G["PartyMemberFrame"..i]:UnregisterAllEvents()
		_G["PartyMemberFrame"..i.."PetFrame"]:UnregisterAllEvents()
		_G["PartyMemberFrame"..i]:Hide()
	end
end

-- cheap way to get support distances
function StellarUF:InitSpellRange()
	if self.cls == "PALADIN" then
		self.spell30 = "Purify"
		self.spell40 = "Holy Light"
	elseif self.cls == "PRIEST" then
		self.spell30 = "Cure Disease"
		self.spell40 = "Lesser Heal"
	elseif self.cls == "DRUID" then
		self.spell30 = "Cure Poison"
		self.spell40 = "Healing Touch"
	elseif self.cls == "MAGE" then
		self.spell30 = "Remove Lesser Curse"
	elseif self.cls == "SHAMAN" then
		self.spell30 = "Cure Poison"
		self.spell40 = "Healing Wave"
	end
end




