-- just some frame functions to reduce redundant lines of code
function StellarUF:CreateFrame(parent, n, w, h, backdrop, r, g, b, a, mouseable, texture)
	parent[n] = CreateFrame((mouseable and "BUTTON") or "Frame", nil, parent)
	local f = parent[n]
	f:ClearAllPoints()
	f:SetWidth(w)
	f:SetHeight(h)
	
	if backdrop then
		if texture then
			f:SetBackdrop({ bgFile = texture, })
		else
			f:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, })
		end
		f:SetBackdropColor(r or 0, g or 0, b or 0, a or 1)
	end
end

function StellarUF:CreateTexture(parent, n, w, h, overlay, inset, texture)
	parent[n] = parent:CreateTexture(nil, overlay or "BACKGROUND")
	local t = parent[n]
	t:ClearAllPoints()
	
	t:SetWidth(w or 1)
	t:SetHeight(h or 1)
	if inset then
		t:SetPoint("TOPLEFT", parent, inset, -inset)
		t:SetPoint("BOTTOMRIGHT", parent, -inset, inset)
	end

	t:SetTexture(texture)
end

function StellarUF:CreateText(parent, n, w, h, font, fontsize, justifyH, justifyV, point, x, y, overlay, noshadow)
	parent[n] = parent:CreateFontString(nil, overlay or "ARTWORK")
	local t = parent[n]
	t:SetJustifyH(justifyH)
	t:SetJustifyV(justifyV)
	t:SetFont(font, fontsize)
	t:SetHeight(h)
	t:SetWidth(w-2)
	t:ClearAllPoints()
	t:SetPoint(point, parent, point, x, y)
	if not noshadow then
		t:SetShadowColor(0.1, 0.1, 0.1)
		t:SetShadowOffset(1,-1)
	end
	t:SetTextColor(self.reactioncolor[8].r, self.reactioncolor[8].g, self.reactioncolor[8].b)
end


--****************************************************--
----------------
---- Icons -----
----------------

function StellarUF:CreateIcon(parent, n, size, backdrop, mouseable)
	self:CreateFrame(parent, n, size, size, backdrop, 0, 0, 0, 0.5, mouseable)
	self:CreateTexture(parent[n], "texture", size, size, "OVERLAY", 0, nil)
end

function StellarUF:CreateAuraIcon(parent, n, size, type, id, notooltip)
	self:CreateFrame(parent, n, size, size, true, 0, 0, 0, 1, true, self.aurabg)
	local f = parent[n]
	self:CreateTexture(f, "icon", size-2, size-2, "ARTWORK", 1, nil)
	f.icon:ClearAllPoints()
	f.icon:SetPoint("BOTTOMLEFT")
	f.icon:SetPoint("TOPRIGHT", f, -1.2, -1.2)
	f.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	self:CreateText(f, "count", size, size*(5/8), self.fontnum, size*(5/8), "RIGHT", "BOTTOM", "BOTTOMRIGHT", 0, 0, "OVERLAY")
	
	f.id = id
	if not notooltip then
		if type == "buff" then
			f:SetScript("OnEnter", function()
				GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT", 8, -16)
				GameTooltip:SetUnitBuff(this:GetParent().unit, this.id)
				GameTooltip:Show()
			end)
		else
			f:SetScript("OnEnter", function()
				GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT", 8, -16)
				GameTooltip:SetUnitDebuff(this:GetParent().unit, this.id)
				GameTooltip:Show()
			end)
		end
		f:SetScript("OnLeave", function() GameTooltip:Hide() end)
	end
end

function StellarUF:CreateAuraGroup(parent, n, size, count, type, rows, cols)
	for i=1,count,1 do
		local index = string.format("%s%d", n, i)
		local tindex = string.format("%s%d", n, i-1)
		self:CreateAuraIcon(parent, index, size, type, i)
		parent[index]:Hide()
		if i ~= 1 then
			local prevrow
			if rows and cols and count > cols then
				prevrow = (math.floor(i/cols) * cols) - cols + 1
			end
			if prevrow and math.fmod(i-1,cols) == 0 then
				parent[index]:SetPoint("TOP", parent[n..prevrow], "BOTTOM", 0, 0)
			else
				parent[index]:SetPoint("LEFT", parent[tindex], "RIGHT", 0, 0)
			end
		end
	end
end


--****************************************************--
-------------------
---- Portraits ----
-------------------

function StellarUF:CreatePortrait(parent, size, type)
	self:CreateFrame(parent, "portrait", size, size)
	local p = parent.portrait
	if type == "full" or type == "medium" then
		p:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		                edgeFile = "", edgeSize = 0, })
		p:SetBackdropColor(0.1, 0.1, 0.1, 0.8)
		p.d3 = CreateFrame("PlayerModel", nil, p)
		p.d3:ClearAllPoints()
		p.d3:SetPoint("TOPLEFT", 0.5, -0.5)
		p.d3:SetPoint("BOTTOMRIGHT", 0.5, -0.5)
		p.d3:SetScript("OnShow", function() this:SetCamera(0) this:SetModelScale(2) end)
		self:CreateText(p, "combattext", size*1.5, size/4, self.fontnum, size/4, "CENTER", "TOP", "TOP", 0, 2, "OVERLAY")
		p.d3:SetFrameLevel(2)
		p.combattext.alpha = 0
		
		self:CreateTexture(p, "glow", size, size, "ARTWORK", 0, "Interface\\AddOns\\StellarUF\\images\\portraitglow")
		p.glow.sign = 1
		p.glow.alpha = 0
		p.glow.duration = 0
		p.glow.elapsed = 0
		p.glow:SetAlpha(0)
		p:SetScript("OnUpdate", function() self:UpdatePortraitGlow() end)
	end
	self:CreateTexture(p, "d2", nil, nil, "BORDER", 0)
end

function StellarUF:SetPortraitUnit(unit, f)
	local p = f.portrait
	if UnitIsVisible(unit) and self.db.profile.showmodels and p.d3 then
		if unit == "target" then
			p.d2:Hide()
		end
		p.d3:SetUnit(unit)
		p.d3:Show()
		p.d3:RefreshUnit()
		p.d3:SetCamera(0)
		p:SetBackdropColor(0.1, 0.1, 0.1, 0.8)
		if strfind(unit, "party") then
			SetPortraitTexture(p.d2, unit)
			p.d2:SetAlpha(0.3)
		end
	else
		if unit == "target" then p.d2:Show() end
		if p.d3 then p.d3:Hide() end
		SetPortraitTexture(p.d2, unit)
		p:SetBackdropColor(0.1, 0.1, 0.1, 0)
		p.d2:SetAlpha(1)
	end
end

function StellarUF:ToggleModels()
	if not self.db.profile.showmodels then
		self.db.profile.showmodels = true
		for u,f in pairs(self.units) do
			local p = f.portrait
			if p.d3 then
				p.d3:Show()
				p:SetBackdropColor(0.1, 0.1, 0.1, 0.8)
			end
			p.d2:SetAlpha(0.4)
			self:SetPortraitUnit(u, f)
		end
		self.units.player.portrait.d2:Hide()
		if UnitExists("target") and UnitIsVisible("target") then 
			self.units.target.portrait.d2:Hide() 
		end
	else
		self.db.profile.showmodels = false
		for u,f in pairs(self.units) do
			local p = f.portrait
			if p.d3 then
				p.d3:Hide()
				p:SetBackdropColor(0.1, 0.1, 0.1, 0)
			end
			p.d2:Show()
			self:SetPortraitUnit(u, f)
			p.d2:SetAlpha(0.9)
		end
	end
end


--****************************************************--
--------------------
---- Statusbars ----
--------------------

function StellarUF:CreateHealthBar(parent, w, h, fontsize, reverse, type)
	local left, right, rev = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT", (not reverse and 1) or -1
	self:CreateStatusBar(parent, "hp", w, h, reverse, type)
	local f = parent.hp
	f:SetAlpha(0.95)
	f.bar:SetVertexColor(self.hpcolor.r, self.hpcolor.g, self.hpcolor.b, 1)
	f:SetBackdropColor(0, self.hpcolor.g/3.5, 0, 0.6)
	if type == "full" or type == "medium" then
		f.backbar:SetVertexColor(1, 0.2, 0.2)
		self:CreateText( f, "nametext", w, fontsize+4, self.fontname, fontsize+4, 
		                 left, "TOP", "TOP"..left, 1 * rev, 1 )
		self:CreateText( f, "infotext", w, fontsize+1, self.fontinfo, fontsize+2,
		                 left, "TOP", "TOP"..left, 2 * rev, -(fontsize+1) )
		self:CreateText( f, "hptext", w, h, self.fontnum, fontsize, 
		                 right, "BOTTOM", "BOTTOM"..right, -1.2 * rev, 1.2, nil, true )
		self:CreateText( f, "statustext", w, fontsize+4, self.fontinfo, fontsize+4,
		                 "CENTER", "CENTER", "BOTTOM", 0, -4 , "OVERLAY")
		f.statustext:SetVertexColor(0.5, 0.5, 0.5, 0.7)
	elseif type == "small" then
		self:CreateText( f, "nametext", w, fontsize+2, self.fontname, fontsize+1, 
		                 left, "TOP", "TOP"..left, 0, 1, nil, true )
		self:CreateText( f, "hptext", w, h, self.fontnum, fontsize, 
		                 "CENTER", "BOTTOM", "CENTER", 0, -2, nil, true )
	elseif type == "tiny" then
		self:CreateText( f, "hptext", w, h, self.fontnum, fontsize, 
		                 "CENTER", "BOTTOM", "CENTER", 0, -1, nil, true)
	end
end

function StellarUF:CreateManaBar(parent, w, h, fontsize, reverse, type)
	local left, right, rev = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT", (not reverse and 1) or -1
	self:CreateStatusBar(parent, "mp", w, h, reverse, type)
	parent.mp:SetAlpha(0.95)
	parent.mp:SetBackdropColor(0.2, 0.2, 0.2, 0.6)
	if type == "full" or type == "medium" then
		parent.mp.backbar:SetVertexColor(0.8, 0.8, 0.8, 0.95)
		self:CreateText( parent.mp, "mptext", w, h, self.fontnum, fontsize, 
		                 right, "BOTTOM", "BOTTOM"..right, -1.2 * rev, 1.2, nil, true )
	end
end

function StellarUF:CreateStatusBar(parent, n, w, h, reverse, type)
	local left, right = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT"
	local texture = self.surface:Fetch(self.db.profile.texture)
	self:CreateFrame(parent, n, w, h, true, 0, 0, 0, 1, nil, texture)
	local f = parent[n]
	self:CreateTexture(f, "bar", w, h, "ARTWORK", nil, texture)
	f.bar:SetPoint(left, f, left, 0, 0)
	if type == "full" or type == "medium" then
		self:CreateTexture(f, "backbar", w, h, "BORDER", nil, texture)
		f.backbar:SetPoint(left, f, left, 0, 0)
		f.backbar.alpha = 0.01
		f.backbar:SetAlpha(0)
		f:SetScript("OnUpdate", function()
			if this.backbar.alpha > 0 then
				this.backbar.alpha = math.max(this.backbar.alpha - arg1, 0)
				this.backbar:SetAlpha(math.min(this.backbar.alpha, 1))
			end
		end)
	end
end

-- use this for changing bar values (adopted from DUF)
--   this way allows for a reverted status bar
function StellarUF:SetBarValue(parent, value, reverse, hp, resetFade)
	if value > 1 then value = 1 end
	local old = math.max(parent.bar:GetWidth(), 0.1)
	local new = math.max(parent:GetWidth() * value, 0.1)
	if parent.backbar and not resetFade and old - new > 0.5 then
		parent.backbar:SetWidth(old)
		parent.backbar.alpha = 1.4
		parent.backbar:SetAlpha(1)
		local oldcoord = old / parent:GetWidth()
		if not reverse then
			parent.backbar:SetTexCoord(0, oldcoord, 0, 1)
		else
			parent.backbar:SetTexCoord(oldcoord, 0, 0, 1)
		end
	end
	parent.bar:SetWidth(new)
	
	if not reverse then 
		parent.bar:SetTexCoord(0, math.max(value,0.001), 0, 1)
	else 
		parent.bar:SetTexCoord(math.max(value,0.001), 0, 0, 1)
	end
	
	-- hp green to red coloring
	if hp then
		local c = self.hpcolor
		local r,g
		if value > 0.5 then
			r = c.r + (((1 - value) * 2) * (c.g - c.r))
			g = c.g
		else
			r = c.g
			g = c.g - ((0.5 - value) * 2 * c.g)
		end
		parent.bar:SetVertexColor(r, g, c.b)
	end
end

--****************************************************--
