-- This file contains all the event/update functions

local tar, pla

function StellarUF:RegisterEvents()
	-- units
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateAll")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdateTarget")
	self:RegisterEvent("PLAYER_FOCUS_CHANGED", "UpdateFocus")
	self:RegisterBucketEvent("UNIT_PET", 0.5, "UpdatePet")
	self:RegisterMetro("tt", function() self:UpdateUnit("targettarget") self:UpdateUnit("focustarget") end, 0.25, self)

	-- statusbars
	self:RegisterBucketEvent("UNIT_HEALTH", 0.2, "UpdateHealth")
	self:RegisterBucketEvent("UNIT_MANA", 0.3, "UpdatePower")
	self:RegisterBucketEvent("UNIT_RAGE", 0.3, "UpdatePower")
	self:RegisterBucketEvent("UNIT_FOCUS", 0.4, "UpdatePower")
	self:RegisterBucketEvent("UNIT_ENERGY", 0.2, "UpdatePower")
	self:RegisterEvent("UNIT_DISPLAYPOWER", "UpdatePowerType")
	
	-- text
	self:RegisterBucketEvent("RAID_ROSTER_UPDATE", 1.5, "UpdateGroupText")
	self:RegisterEvent("UNIT_COMBAT", "UpdateCombatStrings")
	self:RegisterEvent("UNIT_SPELLMISS", "UpdateCombatStrings")
	self:RegisterMetro("combat", self.UpdateCombatText, 0.1, self) -- fader
	
	-- icons
	self:RegisterBucketEvent("UNIT_AURA", 0.5, "UpdateAura")
	self:RegisterBucketEvent("RAID_TARGET_UPDATE", 2, "UpdateRaidTargetIcon")
	self:RegisterMetro("status", self.UpdateStatuses, 0.6, self) -- I'm lazy

	-- portrait
	self:RegisterBucketEvent("UNIT_PORTRAIT_UPDATE", 0.5, "UpdatePortrait")
	
	self:StartMetro("tt")
	self:StartMetro("combat")
	self:StartMetro("status")
	
	tar = self.units.target
	pla = self.units.player
end

-- some utility functions
--
function StellarUF:IsInParty()
	return GetNumPartyMembers() > 0
end

function StellarUF:IsInRaid()
	return GetNumRaidMembers() > 0
end

function StellarUF:IsInGroup()
	return self:IsInParty() or self:IsInRaid()
end

function StellarUF:IsUpdateable(unit, frame)
	return frame and UnitExists(unit) and frame:IsVisible()
end

function StellarUF:ClassColorThis(text, cls)
	if not cls or not RAID_CLASS_COLORS[cls] then return text end
	return string.format("|cff%02x%02x%02x%s|r", RAID_CLASS_COLORS[cls].r*255, RAID_CLASS_COLORS[cls].g*255, RAID_CLASS_COLORS[cls].b*255, text)
end

---------------------------------------------------
--- *** GENERAL UPDATES *** ---
--

-- create party baseframes
function StellarUF:CreateParty()
	if self:IsEventRegistered("PARTY_MEMBERS_CHANGED") then 
		self:UnregisterEvent("PARTY_MEMBERS_CHANGED") 
	end
	if InCombatLockdown() then 
		self:ScheduleLeaveCombatAction("CreateParty") 
		return 
	end
	for i=1,4,1 do
		local p, pp = string.format("party%d", i), string.format("partypet%d", i)
		self:CreateUnitFrame(p, "medium")
		self:CreateUnitFrame(pp, "tiny")
		self.units[pp]:ClearAllPoints()
		self.units[pp]:SetParent(self.units[p])
		self.units[pp]:SetPoint("TOPLEFT", self.units[p], "BOTTOMLEFT", -6, -1)
	end
	for ind in pairs(self.layout) do
		self.layout[ind] = nil
	end
	self:RegisterBucketEvent("PARTY_MEMBERS_CHANGED", 1, "UpdateParty")
	self:UpdateParty()
	self.partyvisible = true
end

-- hide party interface in raid if default option is checked
function StellarUF:RaidPartyUpdate()
	if not self.units.party1 or InCombatLockdown() then return end

	if HIDE_PARTY_INTERFACE == "1" and GetNumRaidMembers() > 0 then
		if self.partyvisible then 
			self:HideParty() 
		end
	elseif not self.partyvisible then
		self:ShowParty()
	end
end

function StellarUF:ShowParty()
	for i=1,4,1 do
		local p, pp = string.format("party%d", i), string.format("partypet%d", i)
		RegisterUnitWatch(self.units[p], false)
		RegisterUnitWatch(self.units[pp], false)
	end
	self.partyvisible = true
end

function StellarUF:HideParty()
	for i=1,4,1 do
		local p, pp = string.format("party%d", i), string.format("partypet%d", i)
		UnregisterUnitWatch(self.units[p])
		UnregisterUnitWatch(self.units[pp])
		self.units[p]:Hide()
		self.units[pp]:Hide()
	end
	self.partyvisible = false
end

-- update target
function StellarUF:UpdateTarget()
	PlaySound("igCreatureNeutralSelect") -- they all sound the same to me
	self:UpdateUnit("target", true)
	self:UpdateUnit("targettarget")
	
	-- reset fade effects
	local p = tar.portrait
	p.combattext.alpha = 0
	p.combattext:SetAlpha(0)
	p.glow.alpha = 0
	p.glow.duration = 0
	p.glow:SetAlpha(0)
end

-- update focus unit
function StellarUF:UpdateFocus()
	self:UpdateUnit("focus")
end

-- update pet bucket
function StellarUF:UpdatePet(units)
	for u in pairs(units) do 
		self:UpdateUnit(u)
	end
end

-- update all party members
function StellarUF:UpdateParty()
	for u,f in pairs(self.units) do
		if strfind(u, "party") then
			self:UpdateUnit(u)	
		end
	end
end

-- update everything for a certain unit
function StellarUF:UpdateUnit(unit, resetFade)
	local f = self.units[unit]
	if not self:IsUpdateable(unit, f) then return end
	
	self:UpdateUnitHealth(unit, f, resetFade)
	self:UpdateUnitPower(unit, f, resetFade)
	self:UpdatePowerType(unit)
	self:UpdateNameText(unit, f)
	self:UpdateInfoText(unit, f)
	self:UpdatePVPIcon(unit, f)
	self:UpdateStatusIcon(unit, f)
	self:UpdateClassIcon(unit, f)
	self:UpdatePartyIcons(unit, f)
	self:UpdateUnitPortrait(unit, f)
	self:UpdateUnitAura(unit, f)
	self:UpdateStatusText(unit, f)
	if unit == "target" then 
		self:UpdateRaidTargetIcon() 
		self:UpdateGroupText()
	end
end

-- update everything for all units
function StellarUF:UpdateAll()
	for u,f in pairs(self.units) do
		self:UpdateUnit(u)
	end
end

-- update status related elements (dead, offline, etc)
function StellarUF:UpdateStatuses()
	for u,f in pairs(self.units) do
		if self:IsUpdateable(u,f) then
			self:UpdateStatusText(u,f)
			self:UpdateNameText(u,f)
			self:UpdateInfoText(u,f)
			self:UpdatePVPIcon(u,f)
			self:UpdateStatusIcon(u,f)
			self:UpdatePartyIcons(u,f)
		end
	end
end

---------------------------------------------------
--- *** STATUSBARS *** ---
--

-- update health with bucket support
function StellarUF:UpdateHealth(units)
	for u in pairs(units) do
		self:UpdateUnitHealth(u, self.units[u])
	end
end

-- update health
function StellarUF:UpdateUnitHealth(unit, f, resetFade)
	if not self:IsUpdateable(unit, f) then return end
	
	local current, total = UnitHealth(unit), UnitHealthMax(unit)
	local perc = math.min(current/total, 1)
	local deficit = current - total
		
	if UnitIsDeadOrGhost(unit) or not UnitIsConnected(unit) then
		self:SetBarValue(f.hp, 0, f.reversed, true, resetFade)
		self:SetHealthText(unit, perc * 100, current, total)
	else			
		self:SetBarValue(f.hp, perc, f.reversed, true, resetFade)
		self:SetHealthText(unit, math.max(perc * 100, 1), current, total, deficit)
	end
end

-- health text handler
function StellarUF:SetHealthText(unit, perc, current, total, deficit)
	local f = self.units[unit]
	if f.frametype == "small" or f.frametype == "tiny" then
		f.hp.hptext:SetText(format(self.formats.perc, perc))
	else
	 	if MobHealth3 and unit == "target" then
	 		current, total = MobHealth3:GetUnitHealth("target", current, total, UnitName("target"), UnitLevel("target"))
	 	end
		if deficit and self.db.profile.deficit then
			f.hp.hptext:SetText(format(self.formats.deficit, deficit, current, total))
		else
			f.hp.hptext:SetText(format(self.formats.hptext, perc, current, total))
		end
	end
end

-- update power with bucket support
function StellarUF:UpdatePower(units)
	for u in pairs(units) do
		self:UpdateUnitPower(u, self.units[u])
	end
end

-- update power value
function StellarUF:UpdateUnitPower(unit, f, resetFade)
	if not self:IsUpdateable(unit, f) then return end
	
	local current, total = UnitMana(unit), UnitManaMax(unit)
	local perc = min(current/total, 1)
	
	if UnitIsDeadOrGhost(unit) or not UnitIsConnected(unit) or total < 0.5 then
		self:SetBarValue(f.mp, 0, f.reversed, false, resetFade)
		if f.mp.mptext then
			f.mp.mptext:SetText(format(self.formats.mptext, 0, total))
		end
	else
		self:SetBarValue(f.mp, perc, f.reversed, false, resetFade)
		if f.mp.mptext then
			f.mp.mptext:SetText(format(self.formats.mptext, current, total))
		end
	end
end

function StellarUF:UpdatePowerType(unit)
	local f = self.units[unit]
	if not self:IsUpdateable(unit, f) then return end
	
	local pcolor = self.powercolor[UnitPowerType(unit)]
	f.mp.bar:SetVertexColor(pcolor.r, pcolor.g, pcolor.b)
	f.mp:SetBackdropColor(pcolor.r/3, pcolor.g/3, pcolor.b/3, 0.6)
	self:UpdateUnitPower(unit, f, true)
end

---------------------------------------------------
--- *** PORTRAIT *** ---
--

function StellarUF:UpdatePortrait(units)
	for u in pairs(units) do
		local f = self.units[u]
		if self:IsUpdateable(u, f) then
			self:UpdateUnitPortrait(u, f)
		end
	end
end

-- update portrait
function StellarUF:UpdateUnitPortrait(unit, f)
	self:SetPortraitUnit(unit, f)
end

-- portrait glow effects
function StellarUF:UpdatePortraitGlow()
	if not self:IsUpdateable(this:GetParent().unit, this:GetParent()) then return end
	local g = this.glow
	if g.duration > 0 then
		g.duration = g.duration - arg1
		g.elapsed = g.elapsed + arg1
		if g.elapsed > 0.6 then
			g.sign = -g.sign
			g.elapsed = 0
		end
		g.alpha = math.min(1, g.alpha + (g.sign * arg1))
		g.alpha = math.max(0.4, g.alpha)
		g:SetAlpha(g.alpha)
	else
		g:SetAlpha(0)
	end
end

---------------------------------------------------
--- *** TEXT *** ---
--

-- update unit_combat events, based on DUF's code
function StellarUF:UpdateCombatStrings()
	local f = self.units[arg1]
	if not self:IsUpdateable(arg1, f) or not f.portrait.combattext then return end
	
	local color, value, symbol = "", "", ""
	if arg2 == "IMMUNE" then
		color = "|cFFFFFF00"
		value = CombatFeedbackText["IMMUNE"]
	elseif arg2 == "WOUND" then
		if arg3 == "ABSORB" then 
			color = "|cFFFFFFFF"
			value = TEXT(ABSORB)
		elseif arg4 ~= 0 then
			color = "|cFFFF0000-"
			value = arg4
			if arg3 == "CRITICAL" then 
				symbol = "!"
			elseif arg3 == "CRUSHING" then 
				symbol = "^" 
			end
			if arg4 > UnitLevel("player")*5 and f.portrait.glow then
				f.portrait.glow.duration = 2
				f.portrait.glow:SetVertexColor(1.0, 0.3, 0.3)
			end
		else
			color = "|cFFFFFF00"
			value = CombatFeedbackText["MISS"]
		end
	elseif arg2 == "BLOCK" then
		color = "|cFFFFFF00"
		value = TEXT(BLOCK)
	elseif arg2 == "HEAL" then
		color = "|cFF00FF00+"
		value = arg4
		if arg3 == "CRITICAL" then
			symbol = "!"
		end
		if arg4 > UnitLevel("player")*5 and f.glow then
			f.portrait.glow.duration = 2
			f.portrait.glow:SetVertexColor(0.3, 1.0, 0.3)
		end
	else
		value = CombatFeedbackText[arg2]
	end
	f.portrait.combattext:SetText(string.format("%s%s%s", color, value, symbol))
	f.portrait.combattext.alpha = 2.5
end

-- update fading of combat text
function StellarUF:UpdateCombatText(difference)
	for u,f in pairs(self.units) do
		local ct = f.portrait.combattext
		if self:IsUpdateable(u, f) and ct then
			ct.alpha = max(ct.alpha - difference, 0)
			ct:SetAlpha(min(ct.alpha, 1))
		end
	end
end

-- Name formatting, based on blizz's code
function StellarUF:UpdateNameText(unit, f)
	-- makes the 3d portrait slightly transparent if dead
	if f.portrait.d3 then 
		if UnitIsDeadOrGhost(unit) then 
			f.portrait.d3:SetAlpha(0.6) 
		else 
			f.portrait.d3:SetAlpha(1) 
		end
	end

	if not f.hp.nametext then return end
	local name = GetUnitName(unit)
	local r, g, b
	local dark = 1
	f.hp.nametext:SetText(name)
	
	-- darken name if unit is out of range
	if UnitIsFriend(unit, "player") then
		if unit == "player" or (self.spell30 and IsSpellInRange(self.spell30, unit) == 1) then
			dark = 1
		elseif self.spell40 and IsSpellInRange(self.spell40, unit) == 1 then
			dark = 0.85
		elseif not self.spell30 and not self.spell40 and UnitIsVisible(unit) then
			dark = 1
		else
			dark = 0.7
		end
	end
	if unit == "player" or string.find(unit, "party") or unit == "pet" then
		r,g,b = self.reactioncolor[8].r, self.reactioncolor[8].g, self.reactioncolor[8].b
	elseif UnitPlayerControlled(unit) then
		if UnitCanAttack(unit, "player") then -- Hostile players are red
			if not UnitCanAttack("player", unit) then
				r,g,b = self.reactioncolor[8].r, self.reactioncolor[8].g, self.reactioncolor[8].b
			else
				r,g,b = self.reactioncolor[2].r, self.reactioncolor[2].g, self.reactioncolor[2].b
			end
		elseif UnitCanAttack("player", unit) then -- Players we can attack but which are not hostile are yellow
			r,g,b = self.reactioncolor[4].r, self.reactioncolor[4].g, self.reactioncolor[4].b
		elseif UnitIsPVP(unit) and not UnitIsPVPSanctuary(unit) and not UnitIsPVPSanctuary("player") then -- Players we can assist but are PvP flagged are green
			r,g,b = self.reactioncolor[6].r, self.reactioncolor[6].g, self.reactioncolor[6].b
		else
			r,g,b = self.reactioncolor[8].r, self.reactioncolor[8].g, self.reactioncolor[8].b
		end
	elseif UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
		r,g,b = 0.4, 0.4, 0.4
	else
		local rcolor = self.reactioncolor[UnitReaction(unit, "player") or 8]
		r,g,b = rcolor.r, rcolor.g, rcolor.b
	end
	
	f.hp.nametext:SetTextColor(r*dark, g*dark, b*dark)
end

-- level, creature type text
function StellarUF:UpdateInfoText(unit, f)
	if not f.hp.infotext then return end
	
	local level = UnitLevel(unit)
	local lcls, cls = UnitClass(unit)
	-- color the target's level based on difficulty
	if unit == "target" then
		local hexcolor
		if UnitIsFriend(unit, "player") then
			hexcolor = "|cffeeeeee"
		else
			if level == -1 then 
				level = 999 
			end
			local lcolor = GetDifficultyColor(level)
			hexcolor = string.format("|cFF%02x%02x%02x", lcolor.r * 255, lcolor.g * 255, lcolor.b * 255)
		end
		if level == 999 then 
			level = "??" 
		end
		level = string.format("%s%s%s|r", hexcolor, level, self.classificationtext[UnitClassification(unit)])
			
		-- add creature type and color it by class if unit is player
		local creaturetype = UnitCreatureType(unit) or (UnitIsPlayer(unit) and "Humanoid") or "Unknown"
		if creaturetype == "Not specified" then 
			creaturetype = "Unknown" 
		end
		if UnitIsPlayer(unit) then 
			creaturetype = self:ClassColorThis(creaturetype, cls) 
		end
		level = string.format("%s %s", level, creaturetype)
	elseif lcls then
		level = string.format("%s %s", level, self:ClassColorThis(lcls, cls))
	end
	
	f.hp.infotext:SetText(level)
end

-- raid group numbers
local foundplayer, foundtarget
function StellarUF:UpdateGroupText()
	if self:IsInRaid() then
		foundplayer, foundtarget = nil, nil
		if not UnitInRaid("target") then 
			foundtarget = true
			tar.group:Hide()
		end
		for i=1,MAX_RAID_MEMBERS,1 do
			local name,_,subgroup,_,_,_,_,_,_ = GetRaidRosterInfo(i)
			if not name then break end
			if name == UnitName("player") then
				pla.group.text:SetText(subgroup)
				pla.group:Show()
				foundplayer = true
			end
			if name == UnitName("target") then
				tar.group.text:SetText(subgroup)
				tar.group:Show()
				foundtarget = true
			end
			if foundplayer and foundtarget then break end
		end
	else
		pla.group:Hide()
		tar.group:Hide()
	end
end

-- Dead, Offline, Tapped, Civilian text
function StellarUF:UpdateStatusText(unit, f)
	if not f.hp.statustext then return end
	local dead, offline, tapped
	if UnitIsDeadOrGhost(unit) then 
		dead = "DEAD" 
		if f.portrait.glow then
			f.portrait.glow.duration = 1
			f.portrait.glow:SetVertexColor(0.6, 0.6, 0.6)
		end
	end
	if UnitIsPlayer(unit) and not UnitIsConnected(unit) then
		if dead then 
			offline = "/OFFLINE" 
		else 
			offline = "OFFLINE" 
		end
	end
	if unit == "target" and UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
		tapped = "TAPPED"
		self:UpdateNameText("target", f)
	end
	f.hp.statustext:SetText(string.format("%s%s", dead or tapped or "", offline or ""))
end

---------------------------------------------------
--- *** ICONS *** ---
--

-- update aura with bucket support
function StellarUF:UpdateAura(units)
	for unit in pairs(units) do
		self:UpdateUnitAura(unit, self.units[unit])
	end
end

-- buff/debuff icons
function StellarUF:UpdateUnitAura(unit, f)
	if not self:IsUpdateable(unit, f) then return end
	-- look for a dispellable debuff
	if f.debuffalert then 
		f.debuffalert:Hide()
		if UnitIsFriend(unit, "player") then
			for i=1,40,1 do
				local debuff,_,icon,count,type = UnitDebuff(unit, i, true)
				if not debuff then break end
				if type and DebuffTypeColor[type] then
					f.debuffalert.icon:SetTexture(icon)
					f.debuffalert:SetBackdropColor(DebuffTypeColor[type].r, DebuffTypeColor[type].g, DebuffTypeColor[type].b)
					f.debuffalert:Show()
					if count > 1 then
						f.debuffalert.count:SetText(count)
						f.debuffalert.count:Show()
					else
						f.debuffalert.count:Hide()
					end
					break
				end	
			end	
		end
	end
	
	-- don't handle buffs/debuffs for player
	if unit == "player" then return end
	-- update buffs
	local hasBuffs, hasMoreBuffs
	for i=1,32,1 do
		local button = f[string.format("buff%d", i)]
		if not button then break end
		local buff,_,icon,count = UnitBuff(unit, i)
		if buff then
			button.icon:SetTexture(icon)
			if count > 1 then
				button.count:SetText(count)
				button.count:Show()
			else
				button.count:Hide()
			end
			button:SetBackdropColor(0.1,0.1,0.1)
			button:Show()
			hasBuffs = true
			if i > 16 then hasMoreBuffs = true end
		else
			button:Hide()
		end
	end
	
	-- update debuffs
	for i=1,40,1 do
		local button = f[string.format("debuff%d", i)]
		if not button then break end
		local debuff,_,icon,count,type = UnitDebuff(unit, i)
		if debuff then
			button.icon:SetTexture(icon)
			button:SetBackdropColor(DebuffTypeColor[type or "none"].r, DebuffTypeColor[type or "none"].g, DebuffTypeColor[type or "none"].b)
			if count > 1 then
				button.count:SetText(count)
				button.count:Show()
			else
				button.count:Hide()
			end
			button:Show()
		else
			button:Hide()
		end
	end
		
	-- switch locations of debuffs if no buffs are shown
	if unit == "target" or unit == "targettarget" or unit == "focus" or unit == "focustarget" then
		if hasMoreBuffs and f.buff17 then -- extreme case
			f.debuff1:SetPoint("TOPLEFT", f.buff17, "BOTTOMLEFT", 0, -2)
		elseif hasBuffs then
			f.debuff1:SetPoint("TOPLEFT", f.buff1, "BOTTOMLEFT", 0, -2)
		else
			f.debuff1:SetPoint("TOPLEFT", f.buff1, "TOPLEFT", 0, 0)
		end
	end
end

-- cheap flash effect for dispellable debuffs
function StellarUF:UpdateDebuffAlert()
	this.elapsed = this.elapsed + arg1
	if this.elapsed > 0.6 then
		this.sign = -this.sign
		this.elapsed = 0
	end
	this.alpha = math.min(0.55, this.alpha + (this.sign * arg1))
	this.alpha = math.max(this.alpha, 0.15)
	this:SetAlpha(this.alpha)
end

-- status icon to show when an unit is offline, dead, incombat, or resting 
function StellarUF:UpdateStatusIcon(unit, f)
	if not f.statusicon then return end
	local icon = f.statusicon.texture
	if not UnitIsConnected(unit) then
		icon:SetTexture("Interface\\CharacterFrame\\Disconnect-Icon")
		icon:SetTexCoord(0, 1, 0, 1)
		icon:Show()
	elseif UnitIsDeadOrGhost(unit) then
		icon:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Skull")
		icon:SetTexCoord(0, 1, 0, 1)
		icon:Show()
	elseif UnitAffectingCombat(unit) then
		icon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
		icon:SetTexCoord(.5625, .9, .08, .4375)
		icon:Show()
	elseif unit == "player" and IsResting() then
		icon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
		icon:SetTexCoord(.0625, .4375, .0625, .4375)
		icon:Show()
		if f.portrait.glow then
			f.portrait.glow.duration = 2
			f.portrait.glow:SetVertexColor(0.4, 0.4, 1.0)
		end
	else
		icon:Hide()
	end
end

-- class icon
function StellarUF:UpdateClassIcon(unit, f)
	if not f.classicon then return end
	local icon = f.classicon.texture
	icon:SetTexCoord(0, 1, 0, 1)
	local _,class = UnitClass(unit)
	if not class then
		icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		return
	end
	local l, r, t, b
	if class == "WARRIOR" then l, r, t, b = .25, 0, 0, .25
	elseif class == "MAGE" then l, r, t, b = .5, .25, 0, .25
	elseif class == "ROGUE" then l, r, t, b = .75, .5, 0, .25
	elseif class == "DRUID" then l, r, t, b = 1, .75, 0, .25
	elseif class == "HUNTER" then l, r, t, b = .25, 0, .25, .5
	elseif class == "SHAMAN" then l, r, t, b = .5, .25, .25, .5
	elseif class == "PRIEST" then l, r, t, b = .75, .5, .25, .5
	elseif class == "WARLOCK" then l, r, t, b = 1, .75, .25, .5
	elseif class == "PALADIN" then l, r, t, b = .25, 0, .5, .75 end
	icon:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
	icon:SetTexCoord(r + 0.0175, l - 0.0175, t + 0.018, b - 0.01)	
end

-- class icon tooltip (shows sex, race, and faction)
function StellarUF:ClassIconOnEnter()
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT", 4, -4)
	local u = this:GetParent().unit
	local sex = UnitSex(u) -- 1 for N/A, 2 for Male, 3 for Female
	local _, race = UnitRace(u)
	local npc = not UnitIsPlayer(u)
	local faction = UnitFactionGroup(u)
	
	if sex == 2 then 
		sex = "Male"
	elseif sex == 3 then 
		sex = "Female"
	else 
		sex = "Unknown" 
	end
	
	if faction then 
		faction = string.format("(%s)", faction) 
	end
	if not race and npc then 
		race = "NPC" 
	end
	 
	GameTooltip:SetText(string.format("%s %s %s", sex, race or "", faction or ""), 1, 1, 1) 
	GameTooltip:Show()
end

-- pvp flag icon
function StellarUF:UpdatePVPIcon(unit, f)
	if not f.pvpicon then return end
	local icon = f.pvpicon.texture
	if UnitIsPVPFreeForAll(unit) then 
		icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-FFA")
		icon:Show()
	elseif UnitFactionGroup(unit) and UnitIsPVP(unit) then 
		icon:SetTexture(string.format("Interface\\TargetingFrame\\UI-PVP-%s", UnitFactionGroup(unit)))
		icon:Show()
	else 
		icon:Hide()
	end
end

-- leader icon
function StellarUF:UpdatePartyIcons(unit, f)

	-- leader icon
	if not f.leadericon then return end
	if UnitIsPartyLeader(unit) and self:IsInGroup() then
		f.leadericon.texture:Show()
	else
		f.leadericon.texture:Hide()
	end

	-- loot icon
	local _, lootMaster = GetLootMethod()
	local looter
	if lootMaster == 0 then 
		looter = "player"
	elseif lootMaster then 
		looter = string.format("party%d", lootMaster) 
	end
	if looter and UnitIsUnit(looter, unit) then
		f.looticon.texture:Show()
	else
		f.looticon.texture:Hide()
	end
end

-- raid target symbols
function StellarUF:UpdateRaidTargetIcon()
	local f = tar
	if not self:IsUpdateable("target", f) then return end
	
	local iconID = GetRaidTargetIndex("target")
	if iconID then
		SetRaidTargetIconTexture(f.raidtarget.texture, iconID)
		f.raidtarget:Show()
	else
		f.raidtarget:Hide()
	end 
end

-- combo points
function StellarUF:UpdateCombo()
	local points = GetComboPoints()
	if points > 0 then
		this.texture:SetTexture(string.format("%s%d", self.combo, points))
		this.glow:SetTexture(string.format("%s%dglow", self.combo, points))
		this:Show()
	else
		this:Hide()
	end
end

-- glow for combo points
function StellarUF:UpdateComboGlow()
	this.elapsed = this.elapsed + arg1
	if this.elapsed > 0.7 then
		this.sign = -this.sign
		this.elapsed = 0
	end
	this.alpha = math.min(0.95, this.alpha + (this.sign * arg1))
	this.alpha = math.max(this.alpha, 0.45)
	this.glow:SetAlpha(this.alpha)
end
